// Copyright (C) 2009-2009 Martin Sandve Alnæs
// Licensed under the GNU LGPL Version 3.0.
//
// First added:  2009-01-01
// Last changed: 2009-04-01
//
// This demo program computes a projection in 1D.

#include <dolfin.h>
#include "generated_code/Projection1D.h"

using namespace dolfin;
using namespace Projection1D;

class Weight: public Expression 
{
public:
  void eval(Array<double>& values, const Array<double>& x) const
  {
    values[0] = 1.0;
  }
};

class Source: public Expression 
{
public:
  void eval(Array<double>& values, const Array<double>& x) const
  {
    double dx = x[0] - 0.5;
    values[0] = dx*dx;
  }
};

int main()
{
  // Geometry
  UnitInterval mesh(20);

  // Function spaces
  BilinearForm::TrialSpace V(mesh);
  CoefficientSpace_c C(mesh);
  CoefficientSpace_f F(mesh);

  // Functions
  Weight c;
  Source f;
  Function u(V);

  // Forms
  BilinearForm a(V, V);
  LinearForm L(V);

  // Form coefficients
  a.c = c;
  L.f = f;

  VariationalProblem problem(a, L);
  problem.solve(u);

  // Write solution to file
  File u_file("u.pvd");
  u_file << u;

  // Interpolate f onto the discrete space V
  Source source;
  Function finterpolated(V);
  finterpolated = source; 

  // Write interpolated solution to file
  File f_file("f.pvd");
  f_file << finterpolated;
  
  //plot(u);
  
  return 0;
}

